/*
 * rotating_LED.c
 *
 *  Created on: 16/mar/2014
 *      Author: fb
 *
 *  This program turns on a single green led of LEDG7-LEDG0
 *  and makes its position rotating with a period of around 500 ms.
 *  The activation and direction of the rotation is controlled by the pushbuttons KEY3-KEY1, as follows:
 *  KEY2 stops rotation, KEY3 and KEY1 activate rotation clockwise and counterclockwise respectively
 *  The program is sensitive to the edge originated by the pressure of the pushbuttons KEY3-KEY1.
 *
 */

#include "../Inc/config.h"
#include "system.h"
#include <stdint.h>


/*
 * Function: HexToSevenSeg
 * -----------------------
 * Converts each of the 4 hex digit of a 16-bit number into the corresponding representation on the 7-seg display
 * shown below and assembles them into a 32-bit word that can be used to control a set of 4x 7-seg displays present
 * in DE boards
 *         0
 *       -----
 *     5 |___| 1
 *     4 | 6 | 2
 *       -----
 *         3
 */

uint32_t HexToSevenSeg(unsigned int hex) {
	const uint8_t LUT_HEX_TO_SEVEN_SEG[16] = {
	    0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x07, 0x7F, 0x67, /* 0-9 */
	    0x77, 0x7C, 0x39, 0x5E, 0x79, 0x71 };                       /* A,b,C,d,E,F */

	uint32_t data = LUT_HEX_TO_SEVEN_SEG[hex        & 15]        |
			           (LUT_HEX_TO_SEVEN_SEG[(hex >> 4) & 15] <<  8) |
			           (LUT_HEX_TO_SEVEN_SEG[(hex >> 8) & 15] << 16) |
			           (LUT_HEX_TO_SEVEN_SEG[(hex >>12) & 15] << 24);

    return data;
}
/* ------------------------------------------------------------------------- */

/* delay generation */
#define CYCLES_PER_MS 254 /* hand tuned to achieve around 1 ms resolution (code optimization OFF)*/
void Wait_ms(unsigned int time_ms) {
	int i,j;
	for(j=0; j<time_ms; j++) {
		for(i=0; i<CYCLES_PER_MS; i++) {;}
	}
}
/* ------------------------------------------------------------------------- */


int main() {

	uint8_t green_leds = LEFT_LED_ON;
	//int8_t green_leds = LEFT_LED_ON;
	uint8_t keys;

	enum {LEFT, RIGHT, STOPPED} rotation = STOPPED;

	/* infinite loop */
	while(1) {
		RED_LEDS_DATA_REG = SLIDER_SWITCHES_DATA_REG;

		/* read event on PUSHBUTTON */
		keys = PUHBUTTONS_EDGE_CAPTURE_REG;
		/* clear all events on PUSHBUTTON*/
		PUHBUTTONS_EDGE_CAPTURE_REG = 0xF;
		/* update rotation according to which KEY was pressed */
		if(keys & (1<<KEY_STOP)) {
			rotation = STOPPED;
		} else
			if(keys & (1<<KEY_LEFT)) {
				rotation = LEFT;
				} else
					if (keys & (1<<KEY_RIGHT)) {
						rotation = RIGHT;
					}
		/* update green led status according to rotation direction*/
		switch(rotation) {
			case LEFT:
				green_leds = green_leds << 1;
				if(green_leds == 0) {
					green_leds = RIGHT_LED_ON;
				}
				break;
			case RIGHT:
				green_leds = green_leds >> 1;
				if(green_leds == 0) {
					green_leds = LEFT_LED_ON;
				}
				break;
			default:
				break;
		} // switch(rotation)

		/* change green led status */
		GREEN_LEDS_DATA_REG = green_leds;

		/* wait ROT_PERIOD ms */
		Wait_ms(ROT_PERIOD);
	}// while(1)
}// main
